#! /usr/bin/env node
const chalk = require('chalk');
const program = require('commander');
const updateNotifier = require('update-notifier');
const pkg = require('../package.json');
const Mints = require('../lib/mints');

updateNotifier({ pkg }).notify();

program
  .version(pkg.version, '-v, --version')
  .usage(`${chalk.green('<command>')} [options]`)
  .description(`${chalk.dim.cyan('Hope the mints be able to help you!')}`);

/*
* serve
*/
program
  .command('start')
  .description(`${chalk.dim.green('start a development service')}`)
  .option(
    '-c, --config <file>',
    'specified your config file.'
  )
  .option(
    '-d, --out-dir <path>',
    'set the output directory.default to ./dist.'
  )
  .option(
    '-u, --public-url <path>',
    'set the public URL to serve on.default to "/".'
  )
  .option(
    '-p, --port <port>',
    'set the server port.default "4255".'
  )
  .option(
    '-h, --host <host>',
    'set the server host.default "0.0.0.0".'
  )
  .option(
    '--log-level <level>',
    'set the log level, either "0" (no output), "1" (errors), "2" (warnings + errors) or "3" (all).',
    /^([0-3])$/
  )
  .option(
    '-g, --force-global',
    'use global mints.'
  )
  .action(bundle);

/*
* build
*/
program
  .command('build')
  .description(`${chalk.dim.green('build your application')}`)
  .option(
    '-c, --config <filename>',
    'specified your config file.'
  )
  .option(
    '-d, --out-dir <path>',
    'set the output directory.defaults to "dist".'
  )
  .option(
    '--public-url <url>',
    'set the public URL to serve on.default to "/".'
  )
  .option(
    '--no-minify',
    'disable minification'
  )
  .option(
    '-g, --force-global',
    'use global mints.'
  )
  .action(bundle);

/*
* watch
*/
program
  .command('watch')
  .description(`${chalk.dim.green('listening to application changes')}`)
  .option(
    '-c, --config <filename>',
    'specified your config file.'
  )
  .option(
    '-d, --out-dir <path>',
    'set the output directory.defaults to "dist".'
  )
  .option(
    '--public-url <url>',
    'set the public URL to serve on.default to "/".'
  )
  .option(
    '-g, --force-global',
    'use global mints.'
  )
  .action(bundle);

program
  .on('--help', () => {
    console.log('');
    console.log(`  Run ${chalk.bold.red('mints help <command>')} for more information on specific commands`);
    console.log(`  If you have some questions or suggestions, welcome to contribute.Url: ${chalk.underline.magenta('https://github.com/mintsweet/mints/issues')}`);
    console.log('');
  });

/*
* parse
*/
program
  .parse(process.argv);

/*
* action
*/
function bundle(command) {
  const mints = new Mints(command);

  switch(command.name()) {
    case 'start':
      mints
        .start();
      break;
    case 'build':
      mints
        .build();
      break;
    case 'watch':
      mints
        .watch();
      break;
  }
}
