const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require("mini-css-extract-plugin");

module.exports = class WebpackConifg {
  constructor(mints) {
    this.options = mints.options;
  }

  entry() {
    return path.resolve(this.options.cwd, './src');
  }

  output() {
    return {
      filename: '[name].js',
      path: path.join(this.options.cwd, this.options.outDir),
      publicPath: this.options.publicUrl
    };
  }

  alias() {
    return {
      '@': path.join(this.options.cwd, './src')
    };
  }

  rules() {
    const rules = [
      require('./rules/font')(this.options),
      require('./rules/jsx')(this.options)
    ].concat(require('./rules/css')(this.options), require('./rules/image')(this.options));

    return rules;
  }

  plugins() {
    const plugins = [
      new HtmlWebpackPlugin({
        title: 'Mints App',
        template: path.join(__dirname, './template.ejs') 
      }),
      new MiniCssExtractPlugin({
        filename: "[name].css"
      })
    ];
    return plugins;
  }

  generate() {
    const config = {
      mode: this.options.mode,
      entry: this.entry(),
      output: this.output(),
      resolve: {
        extensions: ['.js', '.json', '.jsx'],
        alias: this.alias()
      },
      module: {
        rules: this.rules()
      },
      optimization: {},
      plugins: this.plugins(),
      performance: {
        maxEntrypointSize: 1000 * 1024,
        hints: this.options.dev ? false : 'warning'
      }
    };
    return config;
  }
}
