const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const LANGS = ['css', 'stylus', 'styl', 'sass', 'scss', 'less'];
const path = require('path');

module.exports = (options) => {
  const cssLoader = {
    loader: 'css-loader'
  };

  const postcssLoader = {
    loader: 'postcss-loader',
    options: {
      config: {
        path: path.join(__dirname, '../postcss.config.js')
      }
    }
  };

  function generate(type) {
    const loaders = [cssLoader, postcssLoader];
    if (type === 'styl') {
      type = 'stylus';
    }

    if (type === 'scss') {
      type ='sass';
    }

    if (type && type !== 'css') {
      loaders.push({
        loader: `${type}-loader`
      });
    }

    if (options.css.extract) {
      loaders.unshift({
        loader: MiniCssExtractPlugin.loader
      });
    } else {
       loaders.unshift({
        loader: 'style-loader'
      });
    }
    return loaders;
  }

  const basic = LANGS.map(lang => {
    return {
      test: new RegExp(`\\.${lang}`),
      include: new RegExp(`\\.${lang}`),
      exclude: new RegExp(`\\.module\\.${lang}`),
      use: generate(lang)
    };
  });

  const module = LANGS.map(lang => {
    return {
      test: new RegExp(`\\.module\\.${lang}`),
      include: new RegExp(`\\.module\\.${lang}`),
      use: generate(lang)
    };
  });

  return [].concat(basic, module);
}