const webpack = require('webpack');
const loadConfig = require('mints-load-config');
const handleOptions = require('./utils/handleOptions');
const logger = require('./logger');
const WebpackConifg = require('./builder');
const Express = require('express');
const WebpackDevMiddleware = require('webpack-dev-middleware');

module.exports = class Mints {
  constructor(command) {
    this.command = command;
    this.options = null;
    this.pending = false;
    this.compiler = null;
  }

  prepare() {
    let configOption;
    if (this.command.config) {
      configOption = loadConfig(this.command.config);
    } else {
      configOption = loadConfig();
    }
    this.options = handleOptions(this.command, configOption);
    logger.setOptions(this.options);
    const config = (new WebpackConifg(this)).generate();
    this.compiler = webpack(config);
  }

  start() {
    this.prepare();
    const app = Express();
    app.use(WebpackDevMiddleware(this.compiler, {
      // webpack-dev-middleware options
    }));
    app.listen(3000, () => console.log('Example app listening on port 3000!'))
  }

  watch() {
    console.log('Watching Project...');
  }

  build() {
    console.log('Building Project...');
  }

  webpackCompile(compiler) {
    return new Promise(async (resolve, reject) => {
      compiler.run((err, stats) => {
        if (err) {
          return reject(err.stack || err);
        } else if (stats.hasErrors()) {
          return reject(stats.toJson().errors);
        }
        resolve();
      });
    });
  }
}
